*
* LROM - ROM LOADER
*
       ORG   $2000
       PUT   $2000
*
NOEXEC EQU   $10							* inhibit exec after loading (0 = NO EXEC)
DNUM   EQU   $12							* assigned drive number from server
ENDADR EQU   $76							* last loaded address
CURLOW EQU   $89							* LOW BYTE OF CURSOR POSITION
BIGFIL EQU   $E6							* TOO BIG FILE FLAG
*
DIMASK EQU   $50							* MASK TO DISABLE INTERRUPTS (FIRQ AND IRQ)
EIMASK EQU   $AF							* MASK TO ENABLE INTERRUPTS
HOKINI EQU   $15E							* FIRST SYSTEM HOOK
HOKEND EQU   $1A8							* LAST SYSTEM HOOK
NAMBUF EQU   $1D1							* fixed by BASIC read filename function which stores in $1d1
RAMINI EQU   $3000						* WHERE TO COPY THE ROM
RAMMAX EQU   $6EFF						* RAM LIMIT TO SEND TO HIGHRAM (16K - 256 BYTES) AS A ROM PROGRAM TO BE RUN
RMAXLD EQU   $6FFF						* RAM LIMIT TO LOAD INTO LOW MEMORY (FOR BASIC ROMs)
BASINI EQU   $8000						* BASIC ROM BEGINNING
SYSERR EQU   $8344						* Show error message (B=error code)
OUTSTR EQU   $90E5						* print string at X+1 to DEVNUM
OUTNUM EQU   $957A			         * print number in D to DEVNUM
OUTCHR EQU   $B54A					   * print char at reg 'A' to DEVNUM
GETNAM EQU   $B7AA						* read file name and length into $1d1
IOERR  EQU   $B84B						* IO ERROR MESSAGE
ROMINI EQU   $C000						* BEGINNING OF ROMs AREA 
*
ENTRY  ORCC  #DIMASK	            	* disable FIRQ, IRQ
		 JSR   <$A5							* peek at next character (AFTER "ROM")
       SUBA  #'N							* TO DETECT IF IT WAS AN 'N' - IF SO DON'T EXEC IT AFTER LOADING
       STA   <NOEXEC						* SAVE DIFFERENCE. 0 = noexec
       BNE   SAVVEC						* IF NOT 'N' JUMP NEXT INSTRUCCION
       JSR   <$9F							* JUMP OVER THAT CHAR (THE 'N')
SAVVEC LDX   #LDTEXT-1					* BYTE BEFORE MESSAGE
		 BSR   PRTSTR						* PRINT STRING
* 	
       LDX   ,S							* GET STACK POINTER TO REG 'X'
       LDU   1,X							* GET 1ST ADDRESS FROM VECTOR TABLE
       STU   DWRVEC  					* SAVE AS VECTOR TO DWREAD
       LDU   3,X							* GET 2ND ADDRESS FROM VECTOR TABLE
       STU   DWWVEC						* SAVE AS VECTOR TO DWWRITE
*
GETN   JSR   GETNAM						* read ROM file name and length into $1d1 = nambuf
       LDX   #NAMBUF-1					* BYTE BEFORE packet buffer start
       CLR   ,X							* zero MSB TO MAKE A 16BIT VALUE for Y
       LDY   ,X++							* length of file name (16 bit)
       BEQ   FAILED						* IF LEN=0 NO FILE ENTERED
       LDB   -1,X							* length of name (8 bit)
*											* Adding .ROM if not termination entered      
		 PSHS  B,X							* save pointer to beginning of name and name length
ISDOT  LDA   ,X+							* get a char from name
       CMPA  #'.							* is it a dot?
       BEQ   ALRDOT						* yes, stop search, quit loop
       DECB									* decrement counter
       BNE   ISDOT						* not end of string, get next char name
       LDD   #$2E52						* get chars ".R"
       STD   ,X++							* add to end of name
       LDD   #$4F4D						* get chars "OM"
       STD   ,X++							* add to end of name
		 LDB   ,S							* get old name length
		 ADDB  #4							* add 4 (because of .VDK added)
		 STB   ,S							* update new length in stack
       STB   NAMBUF						* save new length in byte before string
       LEAY  4,Y                    * correction to name length - DW parameter
ALRDOT PULS  B,X       					* restore pointer and length
       CLR   B,X							* zero terminate name string (for error)
       INC   ,--X							* 1 = DriveWire OP_NAMEOBJ_MOUNT command
       LEAY  2,Y							* length of DW packet = name length + 2
       JSR   (DWWVEC)					* SEND FILE REQUEST TO SERVER
       LDX   #DNUM						* Our drive number variable
       CLR   ,X							* clear it
       LEAY  1,Y							* read one byte back
       JSR   (DWRVEC)					* get drive number
       TST   ,X							* VERIFY RECEIVED VALUE
       BNE   RDECB						* successful mount. IF Z HAS NOT BEEN SET = ERROR (coundn't mount). JUMP TO READIN
       LDB   #$19							* ERRROR CODE FOR MO ERROR
       ANDCC #EIMASK       			* ENABLE INTERRUPTS
       JMP   SYSERR						* SHOW ERROR MESSAGE
*
FAILED ANDCC #EIMASK       			* ENABLE INTERRUPTS
		 JMP   IOERR						* I/O ERRROR MESSAGE
*
PRTSTR LDB	 <CURLOW 					* GET LOW BYTE OF CURSOR POSITION
       ANDB  #$1F							* IS COLUMN ZERO?
       BNE   SHWMSG   					* NO, PRINT WHITH LEADING ENTER
		 LEAX  1,X      					* JUMP LEADING ENTER 
SHWMSG JSR   OUTSTR					   * print string
		 RTS									* RETURN
*
RDECB  LDX   #0							* BEGIN AT SECTOR ZERO
       LDY   #RAMINI						* DESTINATION ADDRESS
       STX   <ENDADR						* SET LAST LOAD ADDRESS TO ZERO 
       CLR   <BIGFIL						* FLAG FOR LONG FILE TO FALSE
ROMLOP LBSR  DOREAD  					* read sector (into Y) 
       BCS   FAILED						* DETECT FRAME ERROR
       BNE   FAILED						* DETECT INCOMPLETE READ
       BLT   NOMORE                 * DETECT END OF FILE. FINISH
       LDA   #'*							* GET 'STAR' ASCII CODE
		 JSR   OUTCHR						* show on screen, we have read a sector
       LEAY  256,Y						* POINT TO NEXT SECTOR ADDRESS
       CMPY  #RMAXLD						* GOT TO THE RAM LIMIT?
       BHS   BIGEND						* NO, KEEP ON COPYING
       LEAX  1,X							* INCREMENT SECTOR NUMBER
       BRA   ROMLOP						* GO READ NEXT SECTOR
BIGEND INC   <BIGFIL						* BIG FILE FLAG TO TRUE
NOMORE STY   <ENDADR						* SAVE AS LAST LOADED ADDRESS
       LDA   #$08							* GET BACKSPACE ASCII CODE
		 JSR	 OUTCHR						* PRINT ON SCREEN (GO BACK ONE CHAR)
       LDA   #'!							* GET 'BANG' ASCII CODE
		 JSR   OUTCHR						* PUT IT ON SCREEN
		 TST   <BIGFIL						* IS THIS A TOO LONG FILE?
		 BEQ   FINAL						* NO, JUMP MESSAGE
		 LDX   #BIGROM-1					* MESSAGE FOR ROM GREATER THAN 16K-256BYTES
		 BSR   PRTSTR						* print message
FINAL  TST   <NOEXEC						* HAS TO BE RUN?
       BNE   RUNIT						* YES, DO IT
       LDX   #ENDTXT-1					* POINT TO END TEXT
		 BSR   PRTSTR						* print message
		 LDD   <ENDADR						* GET LAST LOADED RAM ADDRESS
		 SUBD  #RAMINI						* SUBTRACT INIT RAM ADDRESS
		 JSR   OUTNUM		 				* PRINT LENGTH 
	    ANDCC #EIMASK						* ENABLE INTERRUPTS
       RTS									* RETURN TO THE CALLER

RUNIT  LDD   >ROMINI						* GET TWO BYTES FROM ROM AREA
		 LDX   #$9669						* CREATE A PATTERN 1001 0110 0110 1001
		 STX   >ROMINI						* STORE INTO TWO DOS-ROM BYTES
		 CMPX  >ROMINI						* HAVE BEEN MODIFIED?
		 BNE   BASIC						* NO, MUST SWITCH TO MAP1 AND COPY BASIC TO UPPER RAM
		 STD   >ROMINI						* PUT BACK TWO SAVED BYTES
		 BRA   MAP1							* GO COPY THE ROM TO UPPER MEMORY
BASIC  LDX   #BASINI						* GET BASIC INITIAL ADDRESS
BAS1   STA   $FFDE						* GO MAP0 (ROM)
       LDD   ,X							* GET 2 BYTES
       STA   $FFDF						* GO MAP1 (RAM)
       STD   ,X++							* STORE 2 BYTES, INCR. POINTER
       CMPX  #ROMINI						* REACHED END OF BASIC?
       BLO   BAS1							* NO, KEEP ON COPYING BASIC
MAP1	 LDX	 #RAMINI						* GET ROM ORIGIN ADDRESS AT RAM AREA
       LDU   #ROMINI						* GET ROM DESTINATION ADDRESS
       STA   $FFDF						* GO MAP1 (RAM) AND STAY THERE
MOVE   LDD   ,X++							* GET TWO BYTES, INC POINTER
       STD   ,U++							* STORE TWO BYTES, INC POINTER
       CMPX  <ENDADR 					* reached last loaded address?
		 BHS   NOHOOK						* YES, QUIT LOOP
		 CMPX  #RAMMAX						* reached max ROM possible (16k-256bytes)?
		 BLO	 MOVE							* NO, KEEP ON COPYING ROM
NOHOOK LDA   #$39							* OPCODE FOR RTS
		 LDX   #HOKINI						* GET INITIAL ADDRESS OF SYSTEM HOOKS
HOOKS  STA   ,X+							* OVERWRITE THEM
       CMPX  #HOKEND						* GOT HOOKS END?
       BLO   HOOKS						* NO, KEEP ON OVERWRITING THEM
*      ANDCC #EIMASK					   * IF ADDED, HANGS THE COMPUTER. NEEDS DOSMATE.BIN
		 JMP   ROMINI						* START LOADED ROM
*
DOREAD LDA   <DNUM						* our drive number
       CLRB									* LSN bits 23-16
       PSHS  A,B,X,Y						* SAVE REGISTERS / DATA
       LDA   #$D2							* DW OP_READEX (READ ONE SECTOR)
REREAD PSHS  A								* SAVE COMMAND TO STACK
       LEAX  ,S							* POINT X TO TOP OF STACK
       LDY   #5							* 5 BYTES
       JSR   (DWWVEC)					* SEND 5 BYTES TO DW	  - READEX, DNUM, $0, SECTOR
       PULS  A								* DISCARD COMMAND
       LDX   4,S							* get read buffer pointer (old Y)
       LDY   #256							* read 256 bytes
       LDD   #133							* 1 second timeout
       JSR   (DWRVEC)					* RECEIVE 256 BYTES
       BCS   READEX						* DETECT FRAMING ERROR
       BNE   READEX						* DETECT NOT ALL BYTES RECEIVED
       PSHS  Y								* SAVE CHECKSUM
       LEAX  ,S							* POINT X TO TOP OF STACK
       LDY   #2							* 2 BYTES
       JSR   (DWWVEC)					* SEND CHECKSUM
       LEAY  1,Y							* AFTER A DWRITE 'Y' RETURNS ZEROED, SO THIS WAY IT BECOMES EQUAL TO ONE
       LDD   #133							* 1 second timeout
       JSR   (DWRVEC)					* READ DW ANSWER
       LEAS  2,S							* DISCARD CHECKSUM
       BCS   READEX						* DETECT FRAMING ERROR
       BNE   READEX						* DETECT NOT ALL BYTES RECEIVED
       LDA   ,S							* GET SERVER ANSWER
       BEQ   READEX						* ZERO=OK, RETURN
       CMPA  #$F3							* ERROR CODE IS #E_CRC?
       BNE   READER						* NOT THIS ONE, GO VERIFY EOF CONDITION
       LDA   #$F2							* COMMAND REREAD. #OP_REREADEX
       CLR   ,S							* CLEAR RECEIVED ANSWER
       BRA   REREAD						* READ AGAIN
READER CMPA  #$F4          			* ERROR CODE IS 'READ PAST END' (EOF)?
		 BNE   NOTEOF						* NO, MARK READ ERROR
       ANDCC #$FC                   * UNSETS THE 'C' AND 'V' FLAGS
       ORCC  #$0C							* SETS 'N' AND 'Z' FLAGS. BOTH PREPARE THE FLAGS TO BE POSITIVE AT 'BLT' TEST
       BRA   READEX						* JUMP OVER NEXT INSTRUCTION, NOT A READ ERROR
NOTEOF COMB									* SETS THE CARRY FLAG (ERROR)
READEX PULS  A,B,X,Y,PC					* RESTORE REGISTERS AND RETURN
*
LDTEXT FCC   / LOADING ROM/,13,0		* INITIAL MESSAGE TO THE USER
ENDTXT FCC   $0D, /ROM LENGTH: /, 0	* MESSAGE TO SHOW ROM LENGTH
BIGROM FCC   $0D, /ROM TOO BIG/,0	* MESSAGE FOR ROMS GREATER THAN 16K-256BYTES
DWRVEC FDB   0								* INDIRECT JUMP TO DRIVEWIRE READ FUNCTION
DWWVEC FDB   0								* INDIRECT JUMP TO DRIVEWIRE WRITE FUNCTION
* 
